@@ -32,12 +32,12 @@ function QueueListModel(parent) {
     // External var's
     self.queueItems = ko.observableArray([]);
     self.totalItems = ko.observable(0);
+    self.deleteItems = ko.observableArray([]);
     self.isMultiEditing = ko.observable(false).extend({ persist: 'queueIsMultiEditing' });
     self.isLoading = ko.observable(false).extend({ rateLimit: 100 });
     self.multiEditItems = ko.observableArray([]);
     self.categoriesList = ko.observableArray([]);
     self.scriptsList = ko.observableArray([]);
-    self.scriptsListLoaded = ko.observable(false);
     self.searchTerm = ko.observable('').extend({ rateLimit: { timeout: 400, method: "notifyWhenChangesStop" } });
     self.paginationLimit = ko.observable(20).extend({ persist: 'queuePaginationLimit' });
     self.pagination = new paginationModel(self);
@@ -75,7 +75,7 @@ function QueueListModel(parent) {
         $.each(data.slots, function() {
             var item = this;
             var existingItem = ko.utils.arrayFirst(self.queueItems(), function(i) {
-                return i.id == item.nzo_id;
+                return i.id === item.nzo_id;
             });
 
             if(existingItem) {
@@ -88,7 +88,7 @@ function QueueListModel(parent) {
         });
 
         // Remove all items if there's any
-        if(itemIds.length == self.paginationLimit()) {
+        if(itemIds.length === self.paginationLimit()) {
             // Replace it, so only 1 Knockout DOM-update!
             self.queueItems(newItems);
             newItems = [];
@@ -97,7 +97,7 @@ function QueueListModel(parent) {
             $.each(itemIds, function() {
                 var id = this.toString();
                 self.queueItems.remove(ko.utils.arrayFirst(self.queueItems(), function(i) {
-                    return i.id == id;
+                    return i.id === id;
                 }));
             });
         }
@@ -148,6 +148,27 @@ function QueueListModel(parent) {
 
     }
 
+    self.triggerRemoveDownload = function(items) {
+        // Show and fill modal
+        self.deleteItems.removeAll()
+
+        // Single or multiple items?
+        if(items.length) {
+            ko.utils.arrayPushAll(self.deleteItems, items)
+        } else {
+            self.deleteItems.push(items)
+        }
+
+        // Show modal or delete right away
+        if(self.parent.confirmDeleteQueue()) {
+            // Open modal if desired
+            $('#modal-delete-queue-job').modal("show")
+        } else {
+            // Otherwise just submit right away
+            $('#modal-delete-queue-job form').submit()
+        }
+    }
+
     // Save pagination state
     self.paginationLimit.subscribe(function(newValue) {
         // Save in config if global
@@ -171,7 +192,7 @@ function QueueListModel(parent) {
     // Searching in queue (rate-limited in decleration)
     self.searchTerm.subscribe(function() {
         // Go back to page 1
-        if(self.pagination.currentPage() != 1) {
+        if(self.pagination.currentPage() !== 1) {
             // This forces a refresh
             self.pagination.moveToPage(1);
         } else {
@@ -183,12 +204,12 @@ function QueueListModel(parent) {
     // Clear searchterm
     self.clearSearchTerm = function(data, event) {
         // Was it escape key or click?
-        if(event.type == 'mousedown' || (event.keyCode && event.keyCode == 27)) {
+        if(event.type === 'mousedown' || (event.keyCode && event.keyCode === 27)) {
             self.isLoading(true)
             self.searchTerm('');
         }
         // Was it click and the field is empty? Then we focus on the field
-        if(event.type == 'mousedown' && self.searchTerm() == '') {
+        if(event.type === 'mousedown' && self.searchTerm() === '') {
             $(event.target).parents('.search-box').find('input[type="text"]').focus()
             return;
         }
@@ -255,7 +276,7 @@ function QueueListModel(parent) {
         // Reset form and remove all checked ones
         $form[0].reset();
         self.multiEditItems.removeAll();
-        $('.delete input[name="multiedit"], #multiedit-checkall').prop({'checked': false, 'indeterminate': false})
+        $('.queue-table input[name="multiedit"], #multiedit-checkall-queue').prop({'checked': false, 'indeterminate': false})
 
         // Is the multi-edit in view?
         if(($form.offset().top + $form.outerHeight(true)) > ($(window).scrollTop()+$(window).height())) {
@@ -285,7 +306,7 @@ function QueueListModel(parent) {
         }
 
         // Update check-all buton state
-        setCheckAllState('#multiedit-checkall', '.queue-table input[name="multiedit"]')
+        setCheckAllState('#multiedit-checkall-queue', '.queue-table input[name="multiedit"]')
         return true;
     }
 
@@ -296,7 +317,7 @@ function QueueListModel(parent) {
 
         // We need to re-evaltuate the state of this check-all
         // Otherwise the 'inderterminate' will be overwritten by the click event!
-        setCheckAllState('#multiedit-checkall', '.queue-table input[name="multiedit"]')
+        setCheckAllState('#multiedit-checkall-queue', '.queue-table input[name="multiedit"]')
 
         // Now we can check what happend
         // For when some are checked, or all are checked (but not partly)
@@ -328,7 +349,7 @@ function QueueListModel(parent) {
             self.doMultiEditUpdate()
         }
         // Set state of all the check-all's
-        setCheckAllState('#multiedit-checkall', '.queue-table input[name="multiedit"]')
+        setCheckAllState('#multiedit-checkall-queue', '.queue-table input[name="multiedit"]')
         return true;
     }
 
@@ -352,14 +373,14 @@ function QueueListModel(parent) {
 
         // All non-category updates need to only happen after a category update
         function nonCatUpdates() {
-            if(newScript != '') {
+            if(newScript !== '') {
                 callAPI({
                     mode: 'change_script',
                     value: strIDs,
                     value2: newScript
                 })
             }
-            if(newPrior != '') {
+            if(newPrior !== '') {
                 callAPI({
                     mode: 'queue',
                     name: 'priority',
@@ -367,7 +388,7 @@ function QueueListModel(parent) {
                     value2: newPrior
                 })
             }
-            if(newProc != '') {
+            if(newProc !== '') {
                 callAPI({
                     mode: 'change_opts',
                     value: strIDs,
@@ -384,13 +405,13 @@ function QueueListModel(parent) {
 
             // Wat a little and do the refresh
             // Only if anything changed!
-            if(newStatus || newProc != '' || newPrior != '' || newScript != '' || newCat != '') {
+            if(newStatus || newProc !== '' || newPrior !== '' || newScript !== '' || newCat !== '') {
                 setTimeout(parent.refresh, 100)
             }
         }
 
         // What is changed?
-        if(newCat != '') {
+        if(newCat !== '') {
             callAPI({
                 mode: 'change_cat',
                 value: strIDs,
@@ -402,42 +423,44 @@ function QueueListModel(parent) {
 
     }
 
-    // Selete all selected
+    // Remove downloads from queue
+    self.removeDownloads = function(form) {
+        // Hide modal and show notification
+        $('#modal-delete-queue-job').modal("hide")
+        showNotification('.main-notification-box-removing')
+
+        var strIDs = '';
+        $.each(self.deleteItems(), function(index) {
+            strIDs = strIDs + this.id + ',';
+        })
+
+        callAPI({
+            mode: 'queue',
+            name: 'delete',
+            del_files: 1,
+            value: strIDs
+        }).then(function(response) {
+            self.queueItems.removeAll(self.deleteItems());
+            self.multiEditItems.removeAll(self.deleteItems())
+            self.parent.refresh();
+            hideNotification()
+        });
+    };
+
+    // Delete all selected
     self.doMultiDelete = function() {
         // Anything selected?
         if(self.multiEditItems().length < 1) return;
 
-        // Need confirm
-        if(!self.parent.confirmDeleteQueue() || confirm(glitterTranslate.removeDown)) {
-            // List all the ID's
-            var strIDs = '';
-            $.each(self.multiEditItems(), function(index) {
-                strIDs = strIDs + this.id + ',';
-            })
-
-            // Show notification
-            showNotification('.main-notification-box-removing-multiple', 0, self.multiEditItems().length)
-
-            // Remove
-            callAPI({
-                mode: 'queue',
-                name: 'delete',
-                del_files: 1,
-                value: strIDs
-            }).then(function(response) {
-                if(response.status) {
-                    // Make sure the queue doesnt flicker and then fade-out
-                    self.isLoading(true)
-                    self.parent.refresh()
-                    // Empty it
-                    self.multiEditItems.removeAll();
-                    // Hide notification
-                    hideNotification()
-                }
-            })
-        }
+        // Trigger modal
+        self.triggerRemoveDownload(self.multiEditItems())
     }
 
+    // Focus on the confirm button
+    $('#modal-delete-queue-job').on("shown.bs.modal", function() {
+        $('#modal-delete-queue-job .btn[type="submit"]').focus()
+    })
+
     // On change of page we need to check all those that were in the list!
     self.queueItems.subscribe(function() {
         // We need to wait until the unit is actually finished rendering
@@ -447,7 +470,7 @@ function QueueListModel(parent) {
             })
 
             // Update check-all buton state
-            setCheckAllState('#multiedit-checkall', '.queue-table input[name="multiedit"]')
+            setCheckAllState('#multiedit-checkall-queue', '.queue-table input[name="multiedit"]')
         }, 100)
     }, null, "arrayChange")
 }
@@ -467,8 +490,8 @@ function QueueModel(parent, data) {
     self.index = ko.observable(data.index);
     self.status = ko.observable(data.status);
     self.labels = ko.observableArray(data.labels);
-    self.isGrabbing = ko.observable(data.status == 'Grabbing' || data.avg_age == '-')
-    self.isFetchingBlocks = data.status == 'Fetching' || data.priority == 'Repair' // No need to update
+    self.isGrabbing = ko.observable(data.status === 'Grabbing' || data.avg_age === '-')
+    self.isFetchingBlocks = data.status === 'Fetching' || data.priority === 'Repair' // No need to update
     self.totalMB = ko.observable(parseFloat(data.mb));
     self.remainingMB = ko.observable(parseFloat(data.mbleft))
     self.missingMB = ko.observable(parseFloat(data.mbmissing))
@@ -479,7 +502,7 @@ function QueueModel(parent, data) {
     self.priority = ko.observable(parent.priorityName[data.priority]);
     self.script = ko.observable(data.script);
     self.unpackopts = ko.observable(parseInt(data.unpackopts)) // UnpackOpts fails if not parseInt'd!
-    self.pausedStatus = ko.observable(data.status == 'Paused');
+    self.pausedStatus = ko.observable(data.status === 'Paused');
     self.timeLeft = ko.observable(data.timeleft);
 
     // Initially empty
@@ -490,7 +513,7 @@ function QueueModel(parent, data) {
     // Color of the progress bar
     self.progressColor = ko.computed(function() {
         // Checking
-        if(self.status() == 'Checking') {
+        if(self.status() === 'Checking') {
             return '#58A9FA'
         }
         // Check for missing data, the value is arbitrary! (2%)
@@ -498,7 +521,7 @@ function QueueModel(parent, data) {
             return '#F8A34E'
         }
         // Set to grey, only when not Force download
-        if((self.parent.parent.downloadsPaused() && self.priority() != 2) || self.pausedStatus()) {
+        if((self.parent.parent.downloadsPaused() && self.priority() !== 2) || self.pausedStatus()) {
             return '#B7B7B7'
         }
         // Nothing
@@ -507,6 +530,9 @@ function QueueModel(parent, data) {
 
     // MB's
     self.progressText = ko.pureComputed(function() {
+        if(self.isGrabbing()) {
+            return glitterTranslate.fetchingURL
+        }
         return (self.totalMB() - self.remainingMB()).toFixed(0) + " MB / " + (self.totalMB() * 1).toFixed(0) + " MB";
     })
 
@@ -526,15 +552,15 @@ function QueueModel(parent, data) {
     })
     self.statusText = ko.computed(function() {
         // Checking
-        if(self.status() == 'Checking') {
+        if(self.status() === 'Checking') {
             return glitterTranslate.checking
         }
         // Grabbing
-        if(self.status() == 'Grabbing') {
+        if(self.status() === 'Grabbing') {
             return glitterTranslate.fetch
         }
         // Pausing status
-        if((self.parent.parent.downloadsPaused() && self.priority() != 2) || self.pausedStatus()) {
+        if((self.parent.parent.downloadsPaused() && self.priority() !== 2) || self.pausedStatus()) {
             return glitterTranslate.paused;
         }
         // Just the time
@@ -544,7 +570,7 @@ function QueueModel(parent, data) {
     // Icon to better show force-priority
     self.queueIcon = ko.computed(function() {
         // Force comes first
-        if(self.priority() == 2) {
+        if(self.priority() === 2) {
             return 'glyphicon-forward'
         }
         if(self.pausedStatus()) {
@@ -558,17 +584,17 @@ function QueueModel(parent, data) {
         switch(param) {
             case 'category':
                 // Exception for *
-                if(self.category() == "*")
+                if(self.category() === "*")
                     return glitterTranslate.defaultText
                 return self.category();
             case 'priority':
                 // Onload-exception
-                if(self.priority() == undefined) return;
-                return ko.utils.arrayFirst(self.parent.priorityOptions(), function(item) { return item.value == self.priority()}).name;
+                if(self.priority() === undefined) return;
+                return ko.utils.arrayFirst(self.parent.priorityOptions(), function(item) { return item.value === self.priority()}).name;
             case 'processing':
                 // Onload-exception
-                if(self.unpackopts() == undefined) return;
-                return ko.utils.arrayFirst(self.parent.processingOptions(), function(item) { return item.value == self.unpackopts()}).name;
+                if(self.unpackopts() === undefined) return;
+                return ko.utils.arrayFirst(self.parent.processingOptions(), function(item) { return item.value === self.unpackopts()}).name;
             case 'scripts':
                 return self.script();
             case 'age':
@@ -584,7 +610,7 @@ function QueueModel(parent, data) {
         self.password(data.password);
         self.index(data.index);
         self.status(data.status)
-        self.isGrabbing(data.status == 'Grabbing' || data.avg_age == '-')
+        self.isGrabbing(data.status === 'Grabbing' || data.avg_age === '-')
         self.totalMB(parseFloat(data.mb));
         self.remainingMB(parseFloat(data.mbleft));
         self.missingMB(parseFloat(data.mbmissing))
@@ -595,12 +621,12 @@ function QueueModel(parent, data) {
         self.priority(parent.priorityName[data.priority]);
         self.script(data.script);
         self.unpackopts(parseInt(data.unpackopts)) // UnpackOpts fails if not parseInt'd!
-        self.pausedStatus(data.status == 'Paused');
+        self.pausedStatus(data.status === 'Paused');
         self.timeLeft(data.timeleft);
 
         // Did the label-list change?
         // Otherwise KO will send updates to all texts during refresh()
-        if(self.rawLabels != data.labels.toString()) {
+        if(self.rawLabels !== data.labels.toString()) {
             // Update
             self.labels(data.labels);
             self.rawLabels = data.labels.toString();
@@ -637,7 +663,7 @@ function QueueModel(parent, data) {
     // Do on change
     self.nameForEdit.subscribe(function(newName) {
         // Anything change or empty?
-        if(!newName || self.name() == newName) return;
+        if(!newName || self.name() === newName) return;
 
         // Rename would abort Direct Unpack, so ask if user is sure
         if(self.direct_unpack() && !confirm(glitterTranslate.renameAbort)) return;
@@ -709,29 +735,5 @@ function QueueModel(parent, data) {
         })
     }
 
-    // Remove 1 download from queue
-    self.removeDownload = function(item, event) {
-        // Confirm and remove
-        if(!self.parent.parent.confirmDeleteQueue() || confirm(glitterTranslate.deleteMsg + ":\n" + item.name() + "\n\n" + glitterTranslate.removeDow1)) {
-            var itemToDelete = this;
 
-            // Show notification
-            showNotification('.main-notification-box-removing')
-
-            callAPI({
-                mode: 'queue',
-                name: 'delete',
-                del_files: 1,
-                value: item.id
-            }).then(function(response) {
-                // Make sure no flickering (if there are more items left) and then remove
-                self.parent.isLoading(self.parent.totalItems() > 1)
-                parent.queueItems.remove(itemToDelete);
-                parent.multiEditItems.remove(function(inList) { return inList.id == itemToDelete.id; })
-                self.parent.parent.refresh();
-                // Hide notifcation
-                hideNotification()
-            });
-        }
-    };
 }
